KVM: MMU: zero caches before entering mmu_lock protected section
Clean the pre-allocated cache pages before entering mmu_lock region. This is safe since the caches are per-vcpu.
Smaller chunks are already zeroed by kmem_cache_zalloc.
~= 0.90% reduction in system time with AIM7 on RHEL3 / 4-vcpu.
Signed-off-by Marcelo Tosatti <[email protected]>
arch/x86/kvm/mmu.c +1 -2
@@ -301,7 +301,7 @@ static int mmu_topup_memory_cache_page(struct kvm_mmu_memory_cache *cache,if (cache->nobjs >= min)return 0 ;while (cache->nobjs <ARRAY_SIZE (cache->objects)) { - page =alloc_page (GFP_KERNEL ); + page =alloc_page (GFP_KERNEL |__GFP_ZERO);if (!page)return -ENOMEM ;set_page_private (page,0 );
@@ -352,7 +352,6 @@ static void *mmu_memory_cache_alloc(struct kvm_mmu_memory_cache *mc,BUG_ON (!mc->nobjs); p = mc->objects[--mc->nobjs]; -memset (p,0 , size);return p; }
Quoted 24 lines · Marcelo Tosatti
Clean the pre-allocated cache pages before entering
mmu_lockregion. This is safe since the caches are per-vcpu.Smaller chunks are already zeroed by
kmem_cache_zalloc.~= 0.90% reduction in system time with AIM7 on RHEL3 / 4-vcpu.
Signed-off-by Marcelo Tosatti <[email protected]>arch/x86/kvm/mmu.c +1
-1@@ -301,7 +301,7 @@ static int mmu_topup_memory_cache_page(struct kvm_mmu_memory_cache *cache,if (cache->nobjs >= min)return 0 ;while (cache->nobjs <ARRAY_SIZE (cache->objects)) { - page =alloc_page (GFP_KERNEL ); + page =alloc_page (GFP_KERNEL |__GFP_ZERO);if (!page)return -ENOMEM ;set_page_private (page,0 );
arch/x86/kvm/mmu.c static void *mmu_memory_cache_alloc(struct kvm_mmu_memory_cache *mc, · line 357
Quoted 7 lines
arch/x86/kvm/mmu.c +0
-1@@ -352,7 +352,6 @@ static void *mmu_memory_cache_alloc(struct kvm_mmu_memory_cache *mc,BUG_ON (!mc->nobjs); p = mc->objects[--mc->nobjs]; -memset (p,0 , size);return p; }
I think we can drop the memset altogether, since we will clear the page in ->prefetch_page() anyway.
Signature · 2 lines
I have a truly marvellous patch that fixes the bug which this signature is too narrow to contain.